// 'Flitzer' with remote controlled PS2 gamepad
//
// Arnoud van Delden - December 2020
//
// Libs:
//   PS2X_lib - Check www.billporter.info for updates and to report bugs
//   Check out PS2X_lib.h for the button defines that may be used in your tests and calls.
//
//   Adafruit AdaFruit MotorShield

#include <Wire.h>
#include <PS2X_lib.h>
#include <Servo.h>
#include <Adafruit_MotorShield.h>

//#define DEBUG

// Various defines...
#define ServoPin 9
#define minAngle 55
#define maxAngle 125
#define MaxSpeed 255
#define FlasherL 0
#define FlasherR 14
#define REARLIGHT 100 // Default read light brightness, with headroom for brake light (=255)

Adafruit_MotorShield MShield = Adafruit_MotorShield(0x60);
Adafruit_DCMotor *Motor = MShield.getMotor(1); // Main motor
Adafruit_DCMotor *Light = MShield.getMotor(2); // Headlights...
Adafruit_DCMotor *RedRearLight = MShield.getMotor(3); // Rear and brake light...
Adafruit_DCMotor *ReverseLight = MShield.getMotor(4); // Reverse light..

PS2X ps2x; // PS2 gamepad Controller Class
int error = 0; 
byte controller_type = 0;

Servo ServoMotor;
int motorspeed = 0;
int lastspeed = 0;
int steerAngle = 90; // Start with default steer angle...
unsigned long lasttime;
unsigned long brakeOffTimer;
volatile unsigned long steps;
boolean FlasherLeft = false, FlasherRight = false, FlasherOn = false, RumbleBrake = false,
        WarningLights = false, BrakeLights = false, autoParking = false, lightsOn = false;

void count() { // Interrupt routine of encoder motor...
  steps++;
}

void driveCar() { // IR controlled car...
   ps2x.read_gamepad(RumbleBrake & BrakeLights, 0); // Read controller, rumble during braking...

  // Servo steering...
  steerAngle = map(ps2x.Analog(PSS_RX), 0, 255, minAngle, maxAngle);
#ifdef DEBUG
    if (steerAngle != 90) {
      Serial.print(" steerAngle: ");
      Serial.println(steerAngle, DEC);
    }
#endif  
  ServoMotor.write(steerAngle);

  // Set motor speed...
  motorspeed = ps2x.Analog(PSS_LY) - 127;
  if (motorspeed<0) {
    // Forward...
    motorspeed = abs(motorspeed);
    motorspeed = map(motorspeed, 0, 128,0, 255);
    Motor->setSpeed(motorspeed);
    Motor->run(FORWARD);
    ReverseLight->setSpeed(0);
#ifdef DEBUG
    if (motorspeed>0) {
      Serial.print("Forward motorspeed: ");
      Serial.println(motorspeed, DEC);
    }
#endif    
  } else {
    // Backward/reverse...
    motorspeed = map(motorspeed, 0, 128,0, 255);
    Motor->run(BACKWARD);
    Motor->setSpeed(motorspeed);
    if (motorspeed>1)
      ReverseLight->setSpeed(255);
    else // Switch off reverse light if not moving...
      ReverseLight->setSpeed(0);
#ifdef DEBUG
    if (motorspeed>0) {
      Serial.print("Reverse motorspeed: ");
      Serial.println(motorspeed, DEC);
    }
#endif   
  }
  if (motorspeed<lastspeed)
    BrakeLights = true;
  lastspeed = motorspeed;

  // Head and rear lights...
  if (ps2x.NewButtonState(PSB_RED)) {
    if (ps2x.Button(PSB_RED)) {
      // Red circle was just pressed...
      if (lightsOn) {
        Light->setSpeed(0);
        RedRearLight->setSpeed(0);
        lightsOn=false;
      } else {
        Light->setSpeed(255);
        RedRearLight->setSpeed(REARLIGHT);
        lightsOn=true;
      }
#ifdef DEBUG
      Serial.println("Headlights toggle");
#endif
    }
  }

  // Toggle RumbleBrake...
  if (ps2x.NewButtonState(PSB_PINK)) {
    if (ps2x.Button(PSB_PINK)) {
      // Pink square was just pressed...
      if (RumbleBrake) {
        RumbleBrake=false;
      } else {
        RumbleBrake=true;
      }
#ifdef DEBUG
      Serial.println("RumbleBrake toggle");
#endif
    }
  }
  
  // Warning lights...
  if (ps2x.NewButtonState(PSB_GREEN)) {
    if (ps2x.Button(PSB_GREEN)) {
      // Green triangle was just pressed...
      if (WarningLights)
        WarningLights = false;
      else
        WarningLights = true;
#ifdef DEBUG
    Serial.println("WarningLights toggle");
#endif
    }
  }

  // Auto-parking procedure...
  if (ps2x.NewButtonState(PSB_BLUE)) {
    if (ps2x.Button(PSB_BLUE)) {
      // Blue X was just pressed...
      autoParking = true;
#ifdef DEBUG
      Serial.println("Auto-parking procedure");
#endif
    }
  }

  delay(50);
}

void setup() {
  Serial.begin(115200); // Start serial monitor
  
  MShield.begin();       // Motor Shield init...
  Wire.setClock(400000); // Set I²C-Frequency to 400 kHz...
  ServoMotor.attach(ServoPin); // Servo with default angle...
  ServoMotor.write(steerAngle); // Default middle...
      
  ReverseLight->setSpeed(0); 
  ReverseLight->run(FORWARD);
  RedRearLight->setSpeed(0); 
  RedRearLight->run(BACKWARD);
  Light->setSpeed(0); 
  Light->run(FORWARD);

  MShield.setPin(FlasherL, false);
  MShield.setPin(FlasherR, false);
    
  steps = 0;
  Motor->run(FORWARD); // Default=Forward...
  Motor->setSpeed(0); 

  // Setup pins and settings: config_gamepad(clock, command, attention, data, Pressures?, Rumble?)
  error = ps2x.config_gamepad(13, 11, 10, 12, true, true);     
  if (error == 0) {
    Serial.println("Found Controller, configured successful");
    controller_type = ps2x.readType();
    if (controller_type==1) {
      Serial.println("DualShock Controller Found");
    } else {
      Serial.println("Unknown Controller type");
    }
  } else {
    Serial.println("No controller found, check wiring");
  }
  
  delay(1000);
}

void Flasher() {
  if (FlasherOn) {
    if (FlasherLeft) MShield.setPin(FlasherL, true);
    if (FlasherRight) MShield.setPin(FlasherR, true);
  }
  else {
    if (FlasherLeft) MShield.setPin(FlasherL, false);
    if (FlasherRight) MShield.setPin(FlasherR, false);
  }
  FlasherOn = !FlasherOn;
}

void loop() {
      
  if (autoParking) { // auto parking...
    // To Do...
    autoParking = false; // End parking procedure...
  } else {
    // Normal loop...
    driveCar(); // Main...

    if (WarningLights) { // Flashers on...
      FlasherRight = true;
      FlasherLeft = true;
      if ((millis()-lasttime) > 500) { // Flasher routine
        lasttime = millis();
        Flasher();
      }
    }
    else { // Flashers off...
      if (steerAngle < 90) { // Steering left
        FlasherLeft = true;
        FlasherRight = false;
        if ((millis()-lasttime) > 500) { // Flasher routine
          lasttime = millis();
          Flasher();
        }
      }
      else if (steerAngle > 90) { // Steering right
        FlasherRight = true;
        FlasherLeft = false;
        if ((millis()-lasttime) > 500) { // Flasher routine
          lasttime = millis();
          Flasher();
        }
      }
      else {
        FlasherRight = false;
        FlasherLeft = false;
        MShield.setPin(FlasherL, false); // Left flasher off
        MShield.setPin(FlasherR, false); // Right flasher off
        FlasherOn = false;
      }
    }

    // Brake light handling and auto-off...
    if (BrakeLights) {
      if (brakeOffTimer <= 0) brakeOffTimer = millis(); // Start timer...
      if ((millis()-brakeOffTimer) > 1000) { // Time-out brake lights...
        BrakeLights = false;
        brakeOffTimer = 0;
      } else {
        RedRearLight->setSpeed(255);         
      }
    } else {
      if (lightsOn)
        RedRearLight->setSpeed(REARLIGHT);
      else
        RedRearLight->setSpeed(0);
   }
 
  }
}
